<?php
/**
 * @name		Table Of Contents CK
 * @package		plg_content_tableofcontentsck
 * @copyright	Copyright (C) 2020. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 * @author		Cedric Keiflin - https://www.template-creator.com - https://www.joomlack.fr
 */

defined('_JEXEC') or die;

use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Language\Text;

class PlgContentTableofcontentsck extends CMSPlugin
{
	private $titles = Array();

	protected $autoloadLanguage = true;

	/**
	 * Plugin that collect every Hx tag and put them in a table.
	 *
	 * @param   string   $context  The context of the content being passed to the plugin.
	 * @param   mixed    &$row     An object with a "text" property or the string to be cloaked.
	 * @param   mixed    &$params  Additional parameters. See {@see PlgContentEmailcloak()}.
	 * @param   integer  $page     Optional page number. Unused. Defaults to zero.
	 *
	 * @return  boolean	True on success.
	 */
	public function onContentPrepare($context, &$row, &$params, $page = 0)
	{
		// Don't run this plugin when the content is being indexed
		if ($context === 'com_finder.indexer')
		{
			return true;
		}

		if (is_object($row))
		{
			$text = &$row->text;
		} else {
			$text = &$row;
		}

		// Simple performance check to determine whether bot should process further.
		if (strpos($text, '{toc') === false && (int)$this->params->get('autoload', 0) !== 1)
		{
			return true;
		}

		// check if the table shall not be loaded (forced by user)
		if (strpos($text, '{notoc}') !== false)
		{
			$text = str_replace('{notoc}', '', $text);
			return true;
		}

		// check if we are in a blog, just remove the tag to make it clean
		if ($context === 'com_content.category')
		{
			$text = str_replace('{toc}', '', $text);
			return true;
		}

		// init the variable to avoid concatenation between different calls
		$this->titles = array();

		$theme = $this->params->get('theme');
		if ($theme && $theme !== '-1') {
			$doc = Factory::getDocument();
			$doc->addStylesheet(Uri::root(true) . '/plugins/content/tableofcontentsck/themes/' . $theme);
		}

		return $this->replace($text, $params, $context);
	}

	protected function replace(&$text, &$params, $context)
	{
		// check if the table shall be loaded automatically and no tag exist
		if ((int)$this->params->get('autoload', 0) === 1 && strpos($text, '{toc') === false) {
			// $searchTags = '(<h([1-6])([^>]*)>(.*?)</h)';
			// $text = preg_replace_callback($searchTags, array($this,'createTagLink'), $text);
			$text = $this->parseHeadings($text);
			$table = $this->createTable();
			$text = $table . $text;
			return true;
		} else {
			$extCall = false; // detect if the tag is inside the article source
			// check if the toc is called from the source content (article or module)
			if (strpos($text, '{toc}') !== false)
			{
				$textToUse = $text;
				$shortcode = '{toc}';
			} else if (strpos($text, '{toc') !== false) {
				$patterns = "#{toc (.*)}#Uis";
				$result = preg_match($patterns, $text, $results);
				$input = Factory::getApplication()->input;
				if (isset($results[1])) {
					if ($results[1] === 'current' && $input->get('view', '') === 'article') {
						$articleId = (int)$input->get('id', '0');
					} else {
						$articleId = (int)$results[1];
					}
					$article = $this->getArticle($articleId);
					if ($article !== false) {
					$textToUse = $article->introtext . $article->fulltext;
					} else {
						$textToUse = '';
					}
					$shortcode = $results[0];
				} else {
					$textToUse = $text;
					$shortcode = '{toc}';
				}
			}

			if (isset($shortcode)) {
				// Any Hx tag
				// $searchTags = '(<h(.?)[^>]+>(.*)</)';
				// $searchTags = '(<h(?:"(?:[^\\\"]|\\\.)*"|\'(?:[^\\\\\']|\\\.)*\'|[^\'">])*>(.*?)</)';
				// $searchTags = '(<h([1-6])([^>]*)>(.*?)</h)';
				// $textToUse = preg_replace_callback($searchTags, array($this,'createTagLink'), $textToUse);
				$textToUse = $this->parseHeadings($textToUse);
				if (empty($this->titles)) {
					$table = '';
				} else {
					$table = $this->createTable();
				}
				if ($extCall === true) {
					$text = str_replace($shortcode, $table, $text);
				} else {
					$text = str_replace($shortcode, $table, $textToUse);
				}
			}
		}

		return true;
	}

	function parseHeadings($text) {
		// Any Hx tag
		// $searchTags = '(<h(.?)[^>]+>(.*)</)';
		// $searchTags = '(<h(?:"(?:[^\\\"]|\\\.)*"|\'(?:[^\\\\\']|\\\.)*\'|[^\'">])*>(.*?)</)';
		$startLevel = $this->params->get('startlevel', '1');
		$endLevel = $this->params->get('endlevel', '6');
		if ($endLevel < $startLevel) $endLevel = $startLevel; // security
		$searchTags = '(<h([' . $startLevel . '-' . $endLevel . '])([^>]*)>(.*?)</h)';
		//$searchTags = '(<h([1-6])([^>]*)>(.*?)</h)';
		$text = preg_replace_callback($searchTags, array($this,'createTagLink'), $text);

		return $text;
	}

	function createTagLink($matches) {
		if (! isset($matches[2])) return;

		$text = $matches[3];
		$attrs = $matches[2];
		$level = $matches[1];
		$anchorId = $text;
		// transliterate to avoid issue with cyrillic characters
		if (function_exists('transliterator_transliterate')) {
			$anchorId = transliterator_transliterate('Any-Latin; Latin-ASCII; Lower()', $anchorId);
		}
		$anchorId = strtolower(preg_replace("/\W|_/i", '', $anchorId));

		// create the collection of titles
		$this->titles[] = array('level' => $level, 'text' => $text, 'anchorId' => $anchorId);

		return '<h' . $level . ' id="' . $anchorId . '" ' . $attrs . '>' . $text . '</h';
	}

	function createTable() {
		$useShowHide = (int)$this->params->get('showhide');
		$html = '<div class="table-of-contents-ck-wrap">';
		if ($useShowHide === 1) $html .= '<div class="table-of-contents-ck-toggler">' . Text::_('PLG_TABLEOFCONTENTSCK_CONTENT') . ' <a href="javascript:void(0)" onclick="let TOCCKTable = this.parentNode.parentNode.querySelector(\'ol.table-of-contents-ck\');let TOCCKtableStatus = TOCCKTable.style.display; TOCCKTable.style.display = (TOCCKtableStatus === \'none\' ? \'\' : \'none\')">[' . Text::_('PLG_TABLEOFCONTENTSCK_SHOW_HIDE') . ']</a></div>';
		$html .= '<ol class="table-of-contents-ck" ' . ($useShowHide === 1 ? 'style="display:none;"' : '') . '>';
		$openedTags = 0;
		foreach ($this->titles as $i => $title) {
			if (isset($this->titles[$i -1])) {
				$prevTitle = $this->titles[$i -1];
				$diffLevel = $title['level'] - $prevTitle['level'];
				if ($diffLevel > 0) {
					$html .= str_repeat('<ol>', abs($diffLevel));
					$openedTags += abs($diffLevel);
				} else if ($diffLevel < 0) {
					$html .= str_repeat('</li>', abs($diffLevel));
					$html .= str_repeat('</ol>', abs($diffLevel));
					$openedTags -= abs($diffLevel);
				}
				if ($diffLevel == 0) $html .= '</li>';
			}
			$html .= '<li><a href="#' . $title['anchorId'] . '" class="scrollTo"> ' . $title['text'] .'</a>';
		}
		
		$html .= str_repeat('</li>', $openedTags);
		$html .= str_repeat('</ol>', $openedTags);
		$html .= '</li>';
		$html .= '</ol>';
		$html .= '</div>';

		return $html;
	}

	private function getArticle($id) {
		if (! $id) return false;

		$app = Factory::getApplication();
		if (version_compare(JVERSION, '4') >= 0) {
			$factory = $app->bootComponent('com_content')->getMVCFactory();

			// Get an instance of the generic articles model
			$articles = $factory->createModel('Articles', 'Site', ['ignore_request' => true]);
		} else {
			// load the content articles file
			$com_path = JPATH_SITE . '/components/com_content/';
			include_once $com_path . 'router.php';
			include_once $com_path . 'helpers/route.php';
			JModelLegacy::addIncludePath($com_path . '/models', 'ContentModel');

			// Get an instance of the generic articles model
			$articles = JModelLegacy::getInstance('Articles', 'ContentModel', array('ignore_request' => true));
		}

		$appParams = $app->getParams();
		// Access filter
		$access = !\Joomla\CMS\Component\ComponentHelper::getParams('com_content')->get('show_noauth');
		$authorised = \Joomla\CMS\Access\Access::getAuthorisedViewLevels(\Joomla\CMS\Factory::getUser()->get('id'));
		$articles->setState('filter.access', $access);

		$articles->setState('params', $appParams);
		$articles->setState('filter.published', 1);
		$articles->setState('filter.article_id', $id);
		$items = $articles->getItems();
		$item = $items[0];

		return $item;
	}
}
